/***************************************************************
 *
 * Copyright (C) 1990-2007, Condor Team, Computer Sciences Department,
 * University of Wisconsin-Madison, WI.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you
 * may not use this file except in compliance with the License.  You may
 * obtain a copy of the License at
 * 
 *    http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ***************************************************************/

#include "condor_common.h"
#include "condor_nfs.h"
#include "condor_debug.h"
#include "condor_syscall_mode.h"
#include "condor_io.h"
#include "condor_sys.h"
#include "condor_mmap.h"
#include "condor_error.h"
#include "startup.h"
#include "syscall_param_sizes.h"
#include "../condor_ckpt/signals_control.h"
#include "condor_open.h"

#ifdef LINK_PVM
// ClassAds needed for start_work_request used by PVM starter/shadow
#include "condor_classad.h"
#endif

/* get rid of the mapping of access to access_euid for the syscall library */
#ifdef access
#	undef access
#endif

int CurrentSysCall;

extern ReliSock *syscall_sock;

/* Ahhh, like drinking beer by the pool side. */
extern "C" {
#if defined( SYS_open )
int
REMOTE_CONDOR_open(char *  path , open_flags_t   flags , long   lastarg)
{
	int	scm;
	int	rval;
	condor_errno_t	terrno;
	sigset_t	omask;

	scm = SetSyscalls( SYS_LOCAL | SYS_UNMAPPED );
	omask = _condor_signals_disable();
	CurrentSysCall = CONDOR_open;

	syscall_sock->encode();
	assert( syscall_sock->code(CurrentSysCall) );
	assert( syscall_sock->code(flags) );
	assert( syscall_sock->code(lastarg) );
	assert( syscall_sock->code(path) );
	assert( syscall_sock->end_of_message() );

	syscall_sock->decode();
	assert( syscall_sock->code(rval) );
	if( rval < 0 ) {
		assert( syscall_sock->code(terrno) );
		assert( syscall_sock->end_of_message() );
		_condor_signals_enable( omask );
		SetSyscalls( scm );
		errno = (int)terrno;
		return rval;
	}
	assert( syscall_sock->end_of_message() );
	_condor_signals_enable( omask );
	SetSyscalls( scm );
	return rval;
}

#endif

#if defined( SYS_close )
int
REMOTE_CONDOR_close(int   fd)
{
	int	scm;
	int	rval;
	condor_errno_t	terrno;
	sigset_t	omask;

	scm = SetSyscalls( SYS_LOCAL | SYS_UNMAPPED );
	omask = _condor_signals_disable();
	CurrentSysCall = CONDOR_close;

	syscall_sock->encode();
	assert( syscall_sock->code(CurrentSysCall) );
	assert( syscall_sock->code(fd) );
	assert( syscall_sock->end_of_message() );

	syscall_sock->decode();
	assert( syscall_sock->code(rval) );
	if( rval < 0 ) {
		assert( syscall_sock->code(terrno) );
		assert( syscall_sock->end_of_message() );
		_condor_signals_enable( omask );
		SetSyscalls( scm );
		errno = (int)terrno;
		return rval;
	}
	assert( syscall_sock->end_of_message() );
	_condor_signals_enable( omask );
	SetSyscalls( scm );
	return rval;
}

#endif

#if defined( SYS_unlink )
int
REMOTE_CONDOR_unlink(char *  path)
{
	int	scm;
	int	rval;
	condor_errno_t	terrno;
	sigset_t	omask;

	scm = SetSyscalls( SYS_LOCAL | SYS_UNMAPPED );
	omask = _condor_signals_disable();
	CurrentSysCall = CONDOR_unlink;

	syscall_sock->encode();
	assert( syscall_sock->code(CurrentSysCall) );
	assert( syscall_sock->code(path) );
	assert( syscall_sock->end_of_message() );

	syscall_sock->decode();
	assert( syscall_sock->code(rval) );
	if( rval < 0 ) {
		assert( syscall_sock->code(terrno) );
		assert( syscall_sock->end_of_message() );
		_condor_signals_enable( omask );
		SetSyscalls( scm );
		errno = (int)terrno;
		return rval;
	}
	assert( syscall_sock->end_of_message() );
	_condor_signals_enable( omask );
	SetSyscalls( scm );
	return rval;
}

#endif

int
REMOTE_CONDOR_rename(char *  from , char *  to)
{
	int	scm;
	int	rval;
	condor_errno_t	terrno;
	sigset_t	omask;

	scm = SetSyscalls( SYS_LOCAL | SYS_UNMAPPED );
	omask = _condor_signals_disable();
	CurrentSysCall = CONDOR_rename;

	syscall_sock->encode();
	assert( syscall_sock->code(CurrentSysCall) );
	assert( syscall_sock->code(to) );
	assert( syscall_sock->code(from) );
	assert( syscall_sock->end_of_message() );

	syscall_sock->decode();
	assert( syscall_sock->code(rval) );
	if( rval < 0 ) {
		assert( syscall_sock->code(terrno) );
		assert( syscall_sock->end_of_message() );
		_condor_signals_enable( omask );
		SetSyscalls( scm );
		errno = (int)terrno;
		return rval;
	}
	assert( syscall_sock->end_of_message() );
	_condor_signals_enable( omask );
	SetSyscalls( scm );
	return rval;
}

int
REMOTE_CONDOR_getwd(char * &  path_name)
{
	int	scm;
	int	rval;
	condor_errno_t	terrno;
	sigset_t	omask;

	scm = SetSyscalls( SYS_LOCAL | SYS_UNMAPPED );
	omask = _condor_signals_disable();
	CurrentSysCall = CONDOR_getwd;

	syscall_sock->encode();
	assert( syscall_sock->code(CurrentSysCall) );
	assert( syscall_sock->end_of_message() );

	syscall_sock->decode();
	assert( syscall_sock->code(rval) );
	if( rval < 0 ) {
		assert( syscall_sock->code(terrno) );
		assert( syscall_sock->end_of_message() );
		_condor_signals_enable( omask );
		SetSyscalls( scm );
		errno = (int)terrno;
		return rval;
	}
	assert( syscall_sock->code(path_name) );
	assert( syscall_sock->end_of_message() );
	_condor_signals_enable( omask );
	SetSyscalls( scm );
	return rval;
}

int
REMOTE_CONDOR_geteuid()
{
	int	scm;
	int	rval;
	condor_errno_t	terrno;
	sigset_t	omask;

	scm = SetSyscalls( SYS_LOCAL | SYS_UNMAPPED );
	omask = _condor_signals_disable();
	CurrentSysCall = CONDOR_geteuid;

	syscall_sock->encode();
	assert( syscall_sock->code(CurrentSysCall) );
	assert( syscall_sock->end_of_message() );

	syscall_sock->decode();
	assert( syscall_sock->code(rval) );
	if( rval < 0 ) {
		assert( syscall_sock->code(terrno) );
		assert( syscall_sock->end_of_message() );
		_condor_signals_enable( omask );
		SetSyscalls( scm );
		errno = (int)terrno;
		return rval;
	}
	assert( syscall_sock->end_of_message() );
	_condor_signals_enable( omask );
	SetSyscalls( scm );
	return rval;
}

int
REMOTE_CONDOR_getegid()
{
	int	scm;
	int	rval;
	condor_errno_t	terrno;
	sigset_t	omask;

	scm = SetSyscalls( SYS_LOCAL | SYS_UNMAPPED );
	omask = _condor_signals_disable();
	CurrentSysCall = CONDOR_getegid;

	syscall_sock->encode();
	assert( syscall_sock->code(CurrentSysCall) );
	assert( syscall_sock->end_of_message() );

	syscall_sock->decode();
	assert( syscall_sock->code(rval) );
	if( rval < 0 ) {
		assert( syscall_sock->code(terrno) );
		assert( syscall_sock->end_of_message() );
		_condor_signals_enable( omask );
		SetSyscalls( scm );
		errno = (int)terrno;
		return rval;
	}
	assert( syscall_sock->end_of_message() );
	_condor_signals_enable( omask );
	SetSyscalls( scm );
	return rval;
}

int
REMOTE_CONDOR_register_ckpt_platform(char *  platform , int   len)
{
	int	scm;
	int	rval;
	condor_errno_t	terrno;
	sigset_t	omask;

	scm = SetSyscalls( SYS_LOCAL | SYS_UNMAPPED );
	omask = _condor_signals_disable();
	CurrentSysCall = CONDOR_register_ckpt_platform;

	syscall_sock->encode();
	assert( syscall_sock->code(CurrentSysCall) );
	assert( syscall_sock->code(len) );
	assert( syscall_sock->code_bytes_bool(platform, len) );
	assert( syscall_sock->end_of_message() );

	syscall_sock->decode();
	assert( syscall_sock->code(rval) );
	if( rval < 0 ) {
		assert( syscall_sock->code(terrno) );
		assert( syscall_sock->end_of_message() );
		_condor_signals_enable( omask );
		SetSyscalls( scm );
		errno = (int)terrno;
		return rval;
	}
	assert( syscall_sock->end_of_message() );
	_condor_signals_enable( omask );
	SetSyscalls( scm );
	return rval;
}

int
REMOTE_CONDOR_register_fs_domain(char *  domain)
{
	int	scm;
	int	rval;
	condor_errno_t	terrno;
	sigset_t	omask;

	scm = SetSyscalls( SYS_LOCAL | SYS_UNMAPPED );
	omask = _condor_signals_disable();
	CurrentSysCall = CONDOR_register_fs_domain;

	syscall_sock->encode();
	assert( syscall_sock->code(CurrentSysCall) );
	assert( syscall_sock->code(domain) );
	assert( syscall_sock->end_of_message() );

	syscall_sock->decode();
	assert( syscall_sock->code(rval) );
	if( rval < 0 ) {
		assert( syscall_sock->code(terrno) );
		assert( syscall_sock->end_of_message() );
		_condor_signals_enable( omask );
		SetSyscalls( scm );
		errno = (int)terrno;
		return rval;
	}
	assert( syscall_sock->end_of_message() );
	_condor_signals_enable( omask );
	SetSyscalls( scm );
	return rval;
}

int
REMOTE_CONDOR_register_uid_domain(char *  domain)
{
	int	scm;
	int	rval;
	condor_errno_t	terrno;
	sigset_t	omask;

	scm = SetSyscalls( SYS_LOCAL | SYS_UNMAPPED );
	omask = _condor_signals_disable();
	CurrentSysCall = CONDOR_register_uid_domain;

	syscall_sock->encode();
	assert( syscall_sock->code(CurrentSysCall) );
	assert( syscall_sock->code(domain) );
	assert( syscall_sock->end_of_message() );

	syscall_sock->decode();
	assert( syscall_sock->code(rval) );
	if( rval < 0 ) {
		assert( syscall_sock->code(terrno) );
		assert( syscall_sock->end_of_message() );
		_condor_signals_enable( omask );
		SetSyscalls( scm );
		errno = (int)terrno;
		return rval;
	}
	assert( syscall_sock->end_of_message() );
	_condor_signals_enable( omask );
	SetSyscalls( scm );
	return rval;
}

int
REMOTE_CONDOR_get_universe(int *  univ_id)
{
	int	scm;
	int	rval;
	condor_errno_t	terrno;
	sigset_t	omask;

	scm = SetSyscalls( SYS_LOCAL | SYS_UNMAPPED );
	omask = _condor_signals_disable();
	CurrentSysCall = CONDOR_get_universe;

	syscall_sock->encode();
	assert( syscall_sock->code(CurrentSysCall) );
	assert( syscall_sock->end_of_message() );

	syscall_sock->decode();
	assert( syscall_sock->code(rval) );
	if( rval < 0 ) {
		assert( syscall_sock->code(terrno) );
		assert( syscall_sock->end_of_message() );
		_condor_signals_enable( omask );
		SetSyscalls( scm );
		errno = (int)terrno;
		return rval;
	}
	assert( syscall_sock->code(univ_id) );
	assert( syscall_sock->end_of_message() );
	_condor_signals_enable( omask );
	SetSyscalls( scm );
	return rval;
}

int
REMOTE_CONDOR_get_username(char *  uname)
{
	int	scm;
	int	rval;
	condor_errno_t	terrno;
	sigset_t	omask;

	scm = SetSyscalls( SYS_LOCAL | SYS_UNMAPPED );
	omask = _condor_signals_disable();
	CurrentSysCall = CONDOR_get_username;

	syscall_sock->encode();
	assert( syscall_sock->code(CurrentSysCall) );
	assert( syscall_sock->end_of_message() );

	syscall_sock->decode();
	assert( syscall_sock->code(rval) );
	if( rval < 0 ) {
		assert( syscall_sock->code(terrno) );
		assert( syscall_sock->end_of_message() );
		_condor_signals_enable( omask );
		SetSyscalls( scm );
		errno = (int)terrno;
		return rval;
	}
	assert( syscall_sock->code(uname) );
	assert( syscall_sock->end_of_message() );
	_condor_signals_enable( omask );
	SetSyscalls( scm );
	return rval;
}

int
REMOTE_CONDOR_extern_name(char *  path , char *  buf , int   bufsize)
{
	int	scm;
	int	rval;
	condor_errno_t	terrno;
	sigset_t	omask;

	scm = SetSyscalls( SYS_LOCAL | SYS_UNMAPPED );
	omask = _condor_signals_disable();
	CurrentSysCall = CONDOR_extern_name;

	syscall_sock->encode();
	assert( syscall_sock->code(CurrentSysCall) );
	assert( syscall_sock->code(bufsize) );
	assert( syscall_sock->code(path) );
	assert( syscall_sock->end_of_message() );

	syscall_sock->decode();
	assert( syscall_sock->code(rval) );
	if( rval < 0 ) {
		assert( syscall_sock->code(terrno) );
		assert( syscall_sock->end_of_message() );
		_condor_signals_enable( omask );
		SetSyscalls( scm );
		errno = (int)terrno;
		return rval;
	}
	assert( syscall_sock->code_bytes_bool(buf, bufsize) );
	assert( syscall_sock->end_of_message() );
	_condor_signals_enable( omask );
	SetSyscalls( scm );
	return rval;
}

int
REMOTE_CONDOR_free_fs_blocks(char *  pathname)
{
	int	scm;
	int	rval;
	condor_errno_t	terrno;
	sigset_t	omask;

	scm = SetSyscalls( SYS_LOCAL | SYS_UNMAPPED );
	omask = _condor_signals_disable();
	CurrentSysCall = CONDOR_free_fs_blocks;

	syscall_sock->encode();
	assert( syscall_sock->code(CurrentSysCall) );
	assert( syscall_sock->code(pathname) );
	assert( syscall_sock->end_of_message() );

	syscall_sock->decode();
	assert( syscall_sock->code(rval) );
	if( rval < 0 ) {
		assert( syscall_sock->code(terrno) );
		assert( syscall_sock->end_of_message() );
		_condor_signals_enable( omask );
		SetSyscalls( scm );
		errno = (int)terrno;
		return rval;
	}
	assert( syscall_sock->end_of_message() );
	_condor_signals_enable( omask );
	SetSyscalls( scm );
	return rval;
}

int
REMOTE_CONDOR_reallyexit(int *  status , struct rusage *  use_p)
{
	int	scm;
	int	rval;
	condor_errno_t	terrno;
	sigset_t	omask;

	scm = SetSyscalls( SYS_LOCAL | SYS_UNMAPPED );
	omask = _condor_signals_disable();
	CurrentSysCall = CONDOR_reallyexit;

	syscall_sock->encode();
	assert( syscall_sock->code(CurrentSysCall) );
	assert( syscall_sock->code(use_p) );
	assert( syscall_sock->code(status) );
	assert( syscall_sock->end_of_message() );

	syscall_sock->decode();
	assert( syscall_sock->code(rval) );
	if( rval < 0 ) {
		assert( syscall_sock->code(terrno) );
		assert( syscall_sock->end_of_message() );
		_condor_signals_enable( omask );
		SetSyscalls( scm );
		errno = (int)terrno;
		return rval;
	}
	assert( syscall_sock->end_of_message() );
	_condor_signals_enable( omask );
	SetSyscalls( scm );
	return rval;
}

int
REMOTE_CONDOR_subproc_status(int   id , int *  status , struct rusage *  use_p)
{
	int	scm;
	int	rval;
	condor_errno_t	terrno;
	sigset_t	omask;

	scm = SetSyscalls( SYS_LOCAL | SYS_UNMAPPED );
	omask = _condor_signals_disable();
	CurrentSysCall = CONDOR_subproc_status;

	syscall_sock->encode();
	assert( syscall_sock->code(CurrentSysCall) );
	assert( syscall_sock->code(id) );
	assert( syscall_sock->code(use_p) );
	assert( syscall_sock->code(status) );
	assert( syscall_sock->end_of_message() );

	syscall_sock->decode();
	assert( syscall_sock->code(rval) );
	if( rval < 0 ) {
		assert( syscall_sock->code(terrno) );
		assert( syscall_sock->end_of_message() );
		_condor_signals_enable( omask );
		SetSyscalls( scm );
		errno = (int)terrno;
		return rval;
	}
	assert( syscall_sock->end_of_message() );
	_condor_signals_enable( omask );
	SetSyscalls( scm );
	return rval;
}

int
REMOTE_CONDOR_image_size(int   kbytes)
{
	int	scm;
	int	rval;
	condor_errno_t	terrno;
	sigset_t	omask;

	scm = SetSyscalls( SYS_LOCAL | SYS_UNMAPPED );
	omask = _condor_signals_disable();
	CurrentSysCall = CONDOR_image_size;

	syscall_sock->encode();
	assert( syscall_sock->code(CurrentSysCall) );
	assert( syscall_sock->code(kbytes) );
	assert( syscall_sock->end_of_message() );

	syscall_sock->decode();
	assert( syscall_sock->code(rval) );
	if( rval < 0 ) {
		assert( syscall_sock->code(terrno) );
		assert( syscall_sock->end_of_message() );
		_condor_signals_enable( omask );
		SetSyscalls( scm );
		errno = (int)terrno;
		return rval;
	}
	assert( syscall_sock->end_of_message() );
	_condor_signals_enable( omask );
	SetSyscalls( scm );
	return rval;
}

int
REMOTE_CONDOR_send_rusage(struct rusage *  use_p)
{
	int	scm;
	int	rval;
	condor_errno_t	terrno;
	sigset_t	omask;

	scm = SetSyscalls( SYS_LOCAL | SYS_UNMAPPED );
	omask = _condor_signals_disable();
	CurrentSysCall = CONDOR_send_rusage;

	syscall_sock->encode();
	assert( syscall_sock->code(CurrentSysCall) );
	assert( syscall_sock->code(use_p) );
	assert( syscall_sock->end_of_message() );

	syscall_sock->decode();
	assert( syscall_sock->code(rval) );
	if( rval < 0 ) {
		assert( syscall_sock->code(terrno) );
		assert( syscall_sock->end_of_message() );
		_condor_signals_enable( omask );
		SetSyscalls( scm );
		errno = (int)terrno;
		return rval;
	}
	assert( syscall_sock->end_of_message() );
	_condor_signals_enable( omask );
	SetSyscalls( scm );
	return rval;
}

int
REMOTE_CONDOR_getrusage(int   who , struct rusage *  use_p)
{
	int	scm;
	int	rval;
	condor_errno_t	terrno;
	sigset_t	omask;

	scm = SetSyscalls( SYS_LOCAL | SYS_UNMAPPED );
	omask = _condor_signals_disable();
	CurrentSysCall = CONDOR_getrusage;

	syscall_sock->encode();
	assert( syscall_sock->code(CurrentSysCall) );
	assert( syscall_sock->code(who) );
	assert( syscall_sock->end_of_message() );

	syscall_sock->decode();
	assert( syscall_sock->code(rval) );
	if( rval < 0 ) {
		assert( syscall_sock->code(terrno) );
		assert( syscall_sock->end_of_message() );
		_condor_signals_enable( omask );
		SetSyscalls( scm );
		errno = (int)terrno;
		return rval;
	}
	assert( syscall_sock->code(use_p) );
	assert( syscall_sock->end_of_message() );
	_condor_signals_enable( omask );
	SetSyscalls( scm );
	return rval;
}

int
REMOTE_CONDOR_startup_info_request(STARTUP_INFO *  s)
{
	int	scm;
	int	rval;
	condor_errno_t	terrno;
	sigset_t	omask;

	scm = SetSyscalls( SYS_LOCAL | SYS_UNMAPPED );
	omask = _condor_signals_disable();
	CurrentSysCall = CONDOR_startup_info_request;

	syscall_sock->encode();
	assert( syscall_sock->code(CurrentSysCall) );
	assert( syscall_sock->end_of_message() );

	syscall_sock->decode();
	assert( syscall_sock->code(rval) );
	if( rval < 0 ) {
		assert( syscall_sock->code(terrno) );
		assert( syscall_sock->end_of_message() );
		_condor_signals_enable( omask );
		SetSyscalls( scm );
		errno = (int)terrno;
		return rval;
	}
	assert( syscall_sock->code(s) );
	assert( syscall_sock->end_of_message() );
	_condor_signals_enable( omask );
	SetSyscalls( scm );
	return rval;
}

int
REMOTE_CONDOR_report_error(char *  msg)
{
	int	scm;
	int	rval;
	condor_errno_t	terrno;
	sigset_t	omask;

	scm = SetSyscalls( SYS_LOCAL | SYS_UNMAPPED );
	omask = _condor_signals_disable();
	CurrentSysCall = CONDOR_report_error;

	syscall_sock->encode();
	assert( syscall_sock->code(CurrentSysCall) );
	assert( syscall_sock->code(msg) );
	assert( syscall_sock->end_of_message() );

	syscall_sock->decode();
	assert( syscall_sock->code(rval) );
	if( rval < 0 ) {
		assert( syscall_sock->code(terrno) );
		assert( syscall_sock->end_of_message() );
		_condor_signals_enable( omask );
		SetSyscalls( scm );
		errno = (int)terrno;
		return rval;
	}
	assert( syscall_sock->end_of_message() );
	_condor_signals_enable( omask );
	SetSyscalls( scm );
	return rval;
}

int
REMOTE_CONDOR_report_file_info(char *  kind , char *  name , int   open_count , int   read_count , int   write_count , int   seek_count , int   read_bytes , int   write_bytes)
{
	int	scm;
	int	rval;
	condor_errno_t	terrno;
	sigset_t	omask;

	scm = SetSyscalls( SYS_LOCAL | SYS_UNMAPPED );
	omask = _condor_signals_disable();
	CurrentSysCall = CONDOR_report_file_info;

	syscall_sock->encode();
	assert( syscall_sock->code(CurrentSysCall) );
	assert( syscall_sock->code(open_count) );
	assert( syscall_sock->code(read_count) );
	assert( syscall_sock->code(write_count) );
	assert( syscall_sock->code(seek_count) );
	assert( syscall_sock->code(read_bytes) );
	assert( syscall_sock->code(write_bytes) );
	assert( syscall_sock->code(name) );
	assert( syscall_sock->code(kind) );
	assert( syscall_sock->end_of_message() );

	syscall_sock->decode();
	assert( syscall_sock->code(rval) );
	if( rval < 0 ) {
		assert( syscall_sock->code(terrno) );
		assert( syscall_sock->end_of_message() );
		_condor_signals_enable( omask );
		SetSyscalls( scm );
		errno = (int)terrno;
		return rval;
	}
	assert( syscall_sock->end_of_message() );
	_condor_signals_enable( omask );
	SetSyscalls( scm );
	return rval;
}

int
REMOTE_CONDOR_report_file_info_new(char *  name , int64_t   open_count , int64_t   read_count , int64_t   write_count , int64_t   seek_count , int64_t   read_bytes , int64_t   write_bytes)
{
	int	scm;
	int	rval;
	condor_errno_t	terrno;
	sigset_t	omask;

	scm = SetSyscalls( SYS_LOCAL | SYS_UNMAPPED );
	omask = _condor_signals_disable();
	CurrentSysCall = CONDOR_report_file_info_new;

	syscall_sock->encode();
	assert( syscall_sock->code(CurrentSysCall) );
	assert( syscall_sock->code(open_count) );
	assert( syscall_sock->code(read_count) );
	assert( syscall_sock->code(write_count) );
	assert( syscall_sock->code(seek_count) );
	assert( syscall_sock->code(read_bytes) );
	assert( syscall_sock->code(write_bytes) );
	assert( syscall_sock->code(name) );
	assert( syscall_sock->end_of_message() );

	syscall_sock->decode();
	assert( syscall_sock->code(rval) );
	if( rval < 0 ) {
		assert( syscall_sock->code(terrno) );
		assert( syscall_sock->end_of_message() );
		_condor_signals_enable( omask );
		SetSyscalls( scm );
		errno = (int)terrno;
		return rval;
	}
	assert( syscall_sock->end_of_message() );
	_condor_signals_enable( omask );
	SetSyscalls( scm );
	return rval;
}

int
REMOTE_CONDOR_get_std_file_info(int   fd , char * &  logical_name)
{
	int	scm;
	int	rval;
	condor_errno_t	terrno;
	sigset_t	omask;

	scm = SetSyscalls( SYS_LOCAL | SYS_UNMAPPED );
	omask = _condor_signals_disable();
	CurrentSysCall = CONDOR_get_std_file_info;

	syscall_sock->encode();
	assert( syscall_sock->code(CurrentSysCall) );
	assert( syscall_sock->code(fd) );
	assert( syscall_sock->end_of_message() );

	syscall_sock->decode();
	assert( syscall_sock->code(rval) );
	if( rval < 0 ) {
		assert( syscall_sock->code(terrno) );
		assert( syscall_sock->end_of_message() );
		_condor_signals_enable( omask );
		SetSyscalls( scm );
		errno = (int)terrno;
		return rval;
	}
	assert( syscall_sock->code(logical_name) );
	assert( syscall_sock->end_of_message() );
	_condor_signals_enable( omask );
	SetSyscalls( scm );
	return rval;
}

int
REMOTE_CONDOR_get_file_info_new(char *  logical_name , char * &  actual_url)
{
	int	scm;
	int	rval;
	condor_errno_t	terrno;
	sigset_t	omask;

	scm = SetSyscalls( SYS_LOCAL | SYS_UNMAPPED );
	omask = _condor_signals_disable();
	CurrentSysCall = CONDOR_get_file_info_new;

	syscall_sock->encode();
	assert( syscall_sock->code(CurrentSysCall) );
	assert( syscall_sock->code(logical_name) );
	assert( syscall_sock->end_of_message() );

	syscall_sock->decode();
	assert( syscall_sock->code(rval) );
	if( rval < 0 ) {
		assert( syscall_sock->code(terrno) );
		assert( syscall_sock->end_of_message() );
		_condor_signals_enable( omask );
		SetSyscalls( scm );
		errno = (int)terrno;
		return rval;
	}
	assert( syscall_sock->code(actual_url) );
	assert( syscall_sock->end_of_message() );
	_condor_signals_enable( omask );
	SetSyscalls( scm );
	return rval;
}

int
REMOTE_CONDOR_get_file_info(char *  logical_name , char * &  actual_url)
{
	int	scm;
	int	rval;
	condor_errno_t	terrno;
	sigset_t	omask;

	scm = SetSyscalls( SYS_LOCAL | SYS_UNMAPPED );
	omask = _condor_signals_disable();
	CurrentSysCall = CONDOR_get_file_info;

	syscall_sock->encode();
	assert( syscall_sock->code(CurrentSysCall) );
	assert( syscall_sock->code(logical_name) );
	assert( syscall_sock->end_of_message() );

	syscall_sock->decode();
	assert( syscall_sock->code(rval) );
	if( rval < 0 ) {
		assert( syscall_sock->code(terrno) );
		assert( syscall_sock->end_of_message() );
		_condor_signals_enable( omask );
		SetSyscalls( scm );
		errno = (int)terrno;
		return rval;
	}
	assert( syscall_sock->code(actual_url) );
	assert( syscall_sock->end_of_message() );
	_condor_signals_enable( omask );
	SetSyscalls( scm );
	return rval;
}

int
REMOTE_CONDOR_std_file_info(int   fd , char * &  logical_name , int *  pipe_fd)
{
	int	scm;
	int	rval;
	condor_errno_t	terrno;
	sigset_t	omask;

	scm = SetSyscalls( SYS_LOCAL | SYS_UNMAPPED );
	omask = _condor_signals_disable();
	CurrentSysCall = CONDOR_std_file_info;

	syscall_sock->encode();
	assert( syscall_sock->code(CurrentSysCall) );
	assert( syscall_sock->code(fd) );
	assert( syscall_sock->end_of_message() );

	syscall_sock->decode();
	assert( syscall_sock->code(rval) );
	if( rval < 0 ) {
		assert( syscall_sock->code(terrno) );
		assert( syscall_sock->end_of_message() );
		_condor_signals_enable( omask );
		SetSyscalls( scm );
		errno = (int)terrno;
		return rval;
	}
	assert( syscall_sock->code(pipe_fd) );
	assert( syscall_sock->code(logical_name) );
	assert( syscall_sock->end_of_message() );
	_condor_signals_enable( omask );
	SetSyscalls( scm );
	return rval;
}

int
REMOTE_CONDOR_file_info(char *  logical_name , int *  fd , char * &  physical_name)
{
	int	scm;
	int	rval;
	condor_errno_t	terrno;
	sigset_t	omask;

	scm = SetSyscalls( SYS_LOCAL | SYS_UNMAPPED );
	omask = _condor_signals_disable();
	CurrentSysCall = CONDOR_file_info;

	syscall_sock->encode();
	assert( syscall_sock->code(CurrentSysCall) );
	assert( syscall_sock->code(logical_name) );
	assert( syscall_sock->end_of_message() );

	syscall_sock->decode();
	assert( syscall_sock->code(rval) );
	if( rval < 0 ) {
		assert( syscall_sock->code(terrno) );
		assert( syscall_sock->end_of_message() );
		_condor_signals_enable( omask );
		SetSyscalls( scm );
		errno = (int)terrno;
		return rval;
	}
	assert( syscall_sock->code(fd) );
	assert( syscall_sock->code(physical_name) );
	assert( syscall_sock->end_of_message() );
	_condor_signals_enable( omask );
	SetSyscalls( scm );
	return rval;
}

int
REMOTE_CONDOR_get_buffer_info(int *  bytes , int *  block_size , int *  prefetch_size)
{
	int	scm;
	int	rval;
	condor_errno_t	terrno;
	sigset_t	omask;

	scm = SetSyscalls( SYS_LOCAL | SYS_UNMAPPED );
	omask = _condor_signals_disable();
	CurrentSysCall = CONDOR_get_buffer_info;

	syscall_sock->encode();
	assert( syscall_sock->code(CurrentSysCall) );
	assert( syscall_sock->end_of_message() );

	syscall_sock->decode();
	assert( syscall_sock->code(rval) );
	if( rval < 0 ) {
		assert( syscall_sock->code(terrno) );
		assert( syscall_sock->end_of_message() );
		_condor_signals_enable( omask );
		SetSyscalls( scm );
		errno = (int)terrno;
		return rval;
	}
	assert( syscall_sock->code(prefetch_size) );
	assert( syscall_sock->code(block_size) );
	assert( syscall_sock->code(bytes) );
	assert( syscall_sock->end_of_message() );
	_condor_signals_enable( omask );
	SetSyscalls( scm );
	return rval;
}

int
REMOTE_CONDOR_get_job_attr(char *  attrname , char * &  expr)
{
	int	scm;
	int	rval;
	condor_errno_t	terrno;
	sigset_t	omask;

	scm = SetSyscalls( SYS_LOCAL | SYS_UNMAPPED );
	omask = _condor_signals_disable();
	CurrentSysCall = CONDOR_get_job_attr;

	syscall_sock->encode();
	assert( syscall_sock->code(CurrentSysCall) );
	assert( syscall_sock->code(attrname) );
	assert( syscall_sock->end_of_message() );

	syscall_sock->decode();
	assert( syscall_sock->code(rval) );
	if( rval < 0 ) {
		assert( syscall_sock->code(terrno) );
		assert( syscall_sock->end_of_message() );
		_condor_signals_enable( omask );
		SetSyscalls( scm );
		errno = (int)terrno;
		return rval;
	}
	assert( syscall_sock->code(expr) );
	assert( syscall_sock->end_of_message() );
	_condor_signals_enable( omask );
	SetSyscalls( scm );
	return rval;
}

int
REMOTE_CONDOR_set_job_attr(char *  attrname , char *  expr)
{
	int	scm;
	int	rval;
	condor_errno_t	terrno;
	sigset_t	omask;

	scm = SetSyscalls( SYS_LOCAL | SYS_UNMAPPED );
	omask = _condor_signals_disable();
	CurrentSysCall = CONDOR_set_job_attr;

	syscall_sock->encode();
	assert( syscall_sock->code(CurrentSysCall) );
	assert( syscall_sock->code(expr) );
	assert( syscall_sock->code(attrname) );
	assert( syscall_sock->end_of_message() );

	syscall_sock->decode();
	assert( syscall_sock->code(rval) );
	if( rval < 0 ) {
		assert( syscall_sock->code(terrno) );
		assert( syscall_sock->end_of_message() );
		_condor_signals_enable( omask );
		SetSyscalls( scm );
		errno = (int)terrno;
		return rval;
	}
	assert( syscall_sock->end_of_message() );
	_condor_signals_enable( omask );
	SetSyscalls( scm );
	return rval;
}

int
REMOTE_CONDOR_constrain(char *  expr)
{
	int	scm;
	int	rval;
	condor_errno_t	terrno;
	sigset_t	omask;

	scm = SetSyscalls( SYS_LOCAL | SYS_UNMAPPED );
	omask = _condor_signals_disable();
	CurrentSysCall = CONDOR_constrain;

	syscall_sock->encode();
	assert( syscall_sock->code(CurrentSysCall) );
	assert( syscall_sock->code(expr) );
	assert( syscall_sock->end_of_message() );

	syscall_sock->decode();
	assert( syscall_sock->code(rval) );
	if( rval < 0 ) {
		assert( syscall_sock->code(terrno) );
		assert( syscall_sock->end_of_message() );
		_condor_signals_enable( omask );
		SetSyscalls( scm );
		errno = (int)terrno;
		return rval;
	}
	assert( syscall_sock->end_of_message() );
	_condor_signals_enable( omask );
	SetSyscalls( scm );
	return rval;
}

int
REMOTE_CONDOR_get_iwd(char * &  path)
{
	int	scm;
	int	rval;
	condor_errno_t	terrno;
	sigset_t	omask;

	scm = SetSyscalls( SYS_LOCAL | SYS_UNMAPPED );
	omask = _condor_signals_disable();
	CurrentSysCall = CONDOR_get_iwd;

	syscall_sock->encode();
	assert( syscall_sock->code(CurrentSysCall) );
	assert( syscall_sock->end_of_message() );

	syscall_sock->decode();
	assert( syscall_sock->code(rval) );
	if( rval < 0 ) {
		assert( syscall_sock->code(terrno) );
		assert( syscall_sock->end_of_message() );
		_condor_signals_enable( omask );
		SetSyscalls( scm );
		errno = (int)terrno;
		return rval;
	}
	assert( syscall_sock->code(path) );
	assert( syscall_sock->end_of_message() );
	_condor_signals_enable( omask );
	SetSyscalls( scm );
	return rval;
}

int
REMOTE_CONDOR_get_ckpt_name(char * &  path)
{
	int	scm;
	int	rval;
	condor_errno_t	terrno;
	sigset_t	omask;

	scm = SetSyscalls( SYS_LOCAL | SYS_UNMAPPED );
	omask = _condor_signals_disable();
	CurrentSysCall = CONDOR_get_ckpt_name;

	syscall_sock->encode();
	assert( syscall_sock->code(CurrentSysCall) );
	assert( syscall_sock->end_of_message() );

	syscall_sock->decode();
	assert( syscall_sock->code(rval) );
	if( rval < 0 ) {
		assert( syscall_sock->code(terrno) );
		assert( syscall_sock->end_of_message() );
		_condor_signals_enable( omask );
		SetSyscalls( scm );
		errno = (int)terrno;
		return rval;
	}
	assert( syscall_sock->code(path) );
	assert( syscall_sock->end_of_message() );
	_condor_signals_enable( omask );
	SetSyscalls( scm );
	return rval;
}

int
REMOTE_CONDOR_get_ckpt_mode(condor_signal_t   sig)
{
	int	scm;
	int	rval;
	condor_errno_t	terrno;
	sigset_t	omask;

	scm = SetSyscalls( SYS_LOCAL | SYS_UNMAPPED );
	omask = _condor_signals_disable();
	CurrentSysCall = CONDOR_get_ckpt_mode;

	syscall_sock->encode();
	assert( syscall_sock->code(CurrentSysCall) );
	assert( syscall_sock->code(sig) );
	assert( syscall_sock->end_of_message() );

	syscall_sock->decode();
	assert( syscall_sock->code(rval) );
	if( rval < 0 ) {
		assert( syscall_sock->code(terrno) );
		assert( syscall_sock->end_of_message() );
		_condor_signals_enable( omask );
		SetSyscalls( scm );
		errno = (int)terrno;
		return rval;
	}
	assert( syscall_sock->end_of_message() );
	_condor_signals_enable( omask );
	SetSyscalls( scm );
	return rval;
}

int
REMOTE_CONDOR_get_ckpt_speed()
{
	int	scm;
	int	rval;
	condor_errno_t	terrno;
	sigset_t	omask;

	scm = SetSyscalls( SYS_LOCAL | SYS_UNMAPPED );
	omask = _condor_signals_disable();
	CurrentSysCall = CONDOR_get_ckpt_speed;

	syscall_sock->encode();
	assert( syscall_sock->code(CurrentSysCall) );
	assert( syscall_sock->end_of_message() );

	syscall_sock->decode();
	assert( syscall_sock->code(rval) );
	if( rval < 0 ) {
		assert( syscall_sock->code(terrno) );
		assert( syscall_sock->end_of_message() );
		_condor_signals_enable( omask );
		SetSyscalls( scm );
		errno = (int)terrno;
		return rval;
	}
	assert( syscall_sock->end_of_message() );
	_condor_signals_enable( omask );
	SetSyscalls( scm );
	return rval;
}

int
REMOTE_CONDOR_get_a_out_name(char * &  path)
{
	int	scm;
	int	rval;
	condor_errno_t	terrno;
	sigset_t	omask;

	scm = SetSyscalls( SYS_LOCAL | SYS_UNMAPPED );
	omask = _condor_signals_disable();
	CurrentSysCall = CONDOR_get_a_out_name;

	syscall_sock->encode();
	assert( syscall_sock->code(CurrentSysCall) );
	assert( syscall_sock->end_of_message() );

	syscall_sock->decode();
	assert( syscall_sock->code(rval) );
	if( rval < 0 ) {
		assert( syscall_sock->code(terrno) );
		assert( syscall_sock->end_of_message() );
		_condor_signals_enable( omask );
		SetSyscalls( scm );
		errno = (int)terrno;
		return rval;
	}
	assert( syscall_sock->code(path) );
	assert( syscall_sock->end_of_message() );
	_condor_signals_enable( omask );
	SetSyscalls( scm );
	return rval;
}

int
REMOTE_CONDOR_pvm_info(char *  response , int   mytid)
{
	int	scm;
	int	rval;
	condor_errno_t	terrno;
	sigset_t	omask;

	scm = SetSyscalls( SYS_LOCAL | SYS_UNMAPPED );
	omask = _condor_signals_disable();
	CurrentSysCall = CONDOR_pvm_info;

	syscall_sock->encode();
	assert( syscall_sock->code(CurrentSysCall) );
	assert( syscall_sock->code(mytid) );
	assert( syscall_sock->code(response) );
	assert( syscall_sock->end_of_message() );

	syscall_sock->decode();
	assert( syscall_sock->code(rval) );
	if( rval < 0 ) {
		assert( syscall_sock->code(terrno) );
		assert( syscall_sock->end_of_message() );
		_condor_signals_enable( omask );
		SetSyscalls( scm );
		errno = (int)terrno;
		return rval;
	}
	assert( syscall_sock->end_of_message() );
	_condor_signals_enable( omask );
	SetSyscalls( scm );
	return rval;
}

int
REMOTE_CONDOR_pvm_task_info(int   v_pid , int   pvm_tid)
{
	int	scm;
	int	rval;
	condor_errno_t	terrno;
	sigset_t	omask;

	scm = SetSyscalls( SYS_LOCAL | SYS_UNMAPPED );
	omask = _condor_signals_disable();
	CurrentSysCall = CONDOR_pvm_task_info;

	syscall_sock->encode();
	assert( syscall_sock->code(CurrentSysCall) );
	assert( syscall_sock->code(v_pid) );
	assert( syscall_sock->code(pvm_tid) );
	assert( syscall_sock->end_of_message() );

	syscall_sock->decode();
	assert( syscall_sock->code(rval) );
	if( rval < 0 ) {
		assert( syscall_sock->code(terrno) );
		assert( syscall_sock->end_of_message() );
		_condor_signals_enable( omask );
		SetSyscalls( scm );
		errno = (int)terrno;
		return rval;
	}
	assert( syscall_sock->end_of_message() );
	_condor_signals_enable( omask );
	SetSyscalls( scm );
	return rval;
}

int
REMOTE_CONDOR_suspended(int   susp)
{
	int	scm;
	int	rval;
	condor_errno_t	terrno;
	sigset_t	omask;

	scm = SetSyscalls( SYS_LOCAL | SYS_UNMAPPED );
	omask = _condor_signals_disable();
	CurrentSysCall = CONDOR_suspended;

	syscall_sock->encode();
	assert( syscall_sock->code(CurrentSysCall) );
	assert( syscall_sock->code(susp) );
	assert( syscall_sock->end_of_message() );

	syscall_sock->decode();
	assert( syscall_sock->code(rval) );
	if( rval < 0 ) {
		assert( syscall_sock->code(terrno) );
		assert( syscall_sock->end_of_message() );
		_condor_signals_enable( omask );
		SetSyscalls( scm );
		errno = (int)terrno;
		return rval;
	}
	assert( syscall_sock->end_of_message() );
	_condor_signals_enable( omask );
	SetSyscalls( scm );
	return rval;
}

int
REMOTE_CONDOR_get_file_stream(char *  file , size_t *  len , unsigned int *  ip_addr , u_short *  port_num)
{
	int	scm;
	int	rval;
	condor_errno_t	terrno;
	sigset_t	omask;

	scm = SetSyscalls( SYS_LOCAL | SYS_UNMAPPED );
	omask = _condor_signals_disable();
	CurrentSysCall = CONDOR_get_file_stream;

	syscall_sock->encode();
	assert( syscall_sock->code(CurrentSysCall) );
	assert( syscall_sock->code(file) );
	assert( syscall_sock->end_of_message() );

	syscall_sock->decode();
	assert( syscall_sock->code(rval) );
	if( rval < 0 ) {
		assert( syscall_sock->code(terrno) );
		assert( syscall_sock->end_of_message() );
		_condor_signals_enable( omask );
		SetSyscalls( scm );
		errno = (int)terrno;
		return rval;
	}
	assert( syscall_sock->code(port_num) );
	assert( syscall_sock->code(ip_addr) );
	assert( syscall_sock->code(len) );
	assert( syscall_sock->end_of_message() );
	_condor_signals_enable( omask );
	SetSyscalls( scm );
	return rval;
}

int
REMOTE_CONDOR_put_file_stream(char *  file , size_t   len , unsigned int *  ip_addr , u_short *  port_num)
{
	int	scm;
	int	rval;
	condor_errno_t	terrno;
	sigset_t	omask;

	scm = SetSyscalls( SYS_LOCAL | SYS_UNMAPPED );
	omask = _condor_signals_disable();
	CurrentSysCall = CONDOR_put_file_stream;

	syscall_sock->encode();
	assert( syscall_sock->code(CurrentSysCall) );
	assert( syscall_sock->code(len) );
	assert( syscall_sock->code(file) );
	assert( syscall_sock->end_of_message() );

	syscall_sock->decode();
	assert( syscall_sock->code(rval) );
	if( rval < 0 ) {
		assert( syscall_sock->code(terrno) );
		assert( syscall_sock->end_of_message() );
		_condor_signals_enable( omask );
		SetSyscalls( scm );
		errno = (int)terrno;
		return rval;
	}
	assert( syscall_sock->code(port_num) );
	assert( syscall_sock->code(ip_addr) );
	assert( syscall_sock->end_of_message() );
	_condor_signals_enable( omask );
	SetSyscalls( scm );
	return rval;
}

int
REMOTE_CONDOR_register_ckpt_server(char *  host)
{
	int	scm;
	int	rval;
	condor_errno_t	terrno;
	sigset_t	omask;

	scm = SetSyscalls( SYS_LOCAL | SYS_UNMAPPED );
	omask = _condor_signals_disable();
	CurrentSysCall = CONDOR_register_ckpt_server;

	syscall_sock->encode();
	assert( syscall_sock->code(CurrentSysCall) );
	assert( syscall_sock->code(host) );
	assert( syscall_sock->end_of_message() );

	syscall_sock->decode();
	assert( syscall_sock->code(rval) );
	if( rval < 0 ) {
		assert( syscall_sock->code(terrno) );
		assert( syscall_sock->end_of_message() );
		_condor_signals_enable( omask );
		SetSyscalls( scm );
		errno = (int)terrno;
		return rval;
	}
	assert( syscall_sock->end_of_message() );
	_condor_signals_enable( omask );
	SetSyscalls( scm );
	return rval;
}

int
REMOTE_CONDOR_register_arch(char *  arch)
{
	int	scm;
	int	rval;
	condor_errno_t	terrno;
	sigset_t	omask;

	scm = SetSyscalls( SYS_LOCAL | SYS_UNMAPPED );
	omask = _condor_signals_disable();
	CurrentSysCall = CONDOR_register_arch;

	syscall_sock->encode();
	assert( syscall_sock->code(CurrentSysCall) );
	assert( syscall_sock->code(arch) );
	assert( syscall_sock->end_of_message() );

	syscall_sock->decode();
	assert( syscall_sock->code(rval) );
	if( rval < 0 ) {
		assert( syscall_sock->code(terrno) );
		assert( syscall_sock->end_of_message() );
		_condor_signals_enable( omask );
		SetSyscalls( scm );
		errno = (int)terrno;
		return rval;
	}
	assert( syscall_sock->end_of_message() );
	_condor_signals_enable( omask );
	SetSyscalls( scm );
	return rval;
}

int
REMOTE_CONDOR_register_opsys(char *  opsys)
{
	int	scm;
	int	rval;
	condor_errno_t	terrno;
	sigset_t	omask;

	scm = SetSyscalls( SYS_LOCAL | SYS_UNMAPPED );
	omask = _condor_signals_disable();
	CurrentSysCall = CONDOR_register_opsys;

	syscall_sock->encode();
	assert( syscall_sock->code(CurrentSysCall) );
	assert( syscall_sock->code(opsys) );
	assert( syscall_sock->end_of_message() );

	syscall_sock->decode();
	assert( syscall_sock->code(rval) );
	if( rval < 0 ) {
		assert( syscall_sock->code(terrno) );
		assert( syscall_sock->end_of_message() );
		_condor_signals_enable( omask );
		SetSyscalls( scm );
		errno = (int)terrno;
		return rval;
	}
	assert( syscall_sock->end_of_message() );
	_condor_signals_enable( omask );
	SetSyscalls( scm );
	return rval;
}

int
REMOTE_CONDOR_register_begin_execution()
{
	int	scm;
	int	rval;
	condor_errno_t	terrno;
	sigset_t	omask;

	scm = SetSyscalls( SYS_LOCAL | SYS_UNMAPPED );
	omask = _condor_signals_disable();
	CurrentSysCall = CONDOR_register_begin_execution;

	syscall_sock->encode();
	assert( syscall_sock->code(CurrentSysCall) );
	assert( syscall_sock->end_of_message() );

	syscall_sock->decode();
	assert( syscall_sock->code(rval) );
	if( rval < 0 ) {
		assert( syscall_sock->code(terrno) );
		assert( syscall_sock->end_of_message() );
		_condor_signals_enable( omask );
		SetSyscalls( scm );
		errno = (int)terrno;
		return rval;
	}
	assert( syscall_sock->end_of_message() );
	_condor_signals_enable( omask );
	SetSyscalls( scm );
	return rval;
}

/***************************************************************
 *
 * Copyright (C) 1990-2007, Condor Team, Computer Sciences Department,
 * University of Wisconsin-Madison, WI.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you
 * may not use this file except in compliance with the License.  You may
 * obtain a copy of the License at
 * 
 *    http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ***************************************************************/

/* Converted case statements to function calls instead. -psilord */

#ifdef LINK_PVM   
	// only include CONDOR_work_request when compiling up the PVM
	// starter.  It is used to send startup job info between the PVM
	// starter and shadow.  we do not want it in the regular condor_syscall_lib
	// because it sends Class Ads, and we do not want to force users to
	// link their STANDARD universe jobs with the classad library. -Todd 12/97
int
REMOTE_CONDOR_work_request(ClassAd *ad, char *&a_out, char *&targ, char *&orig,
	condor_signal_t *kill_sig)
{
	int scm;
	int rval;
	condor_errno_t terrno;
	sigset_t    omask;

	assert( a_out == NULL );
	assert( targ == NULL );
	assert( orig == NULL );

	scm = SetSyscalls( SYS_LOCAL | SYS_UNMAPPED );
	omask = _condor_signals_disable();

	CurrentSysCall = CONDOR_work_request;

	syscall_sock->encode();
	assert( syscall_sock->code(CurrentSysCall) );
	assert( syscall_sock->end_of_message() );

	syscall_sock->decode();
	assert( syscall_sock->code(rval) );
	if( rval < 0 ) {
		assert( syscall_sock->code(terrno) );
		assert( syscall_sock->end_of_message() );
		_condor_signals_enable( omask );
		SetSyscalls( scm );
		errno = (int)terrno;
		return rval;
	}
	assert( syscall_sock->code(kill_sig) );
	assert( syscall_sock->code(orig) );
	assert( syscall_sock->code(targ) );
	assert( syscall_sock->code(a_out) );
	assert( ad->initFromStream(*syscall_sock) );
	assert( syscall_sock->end_of_message() );

	_condor_signals_enable( omask );
	SetSyscalls( scm );
	return rval;
}
#endif

int
REMOTE_CONDOR_send_a_file(char *local, char *remote, mode_t mode)
{
	int		loc_fd;
	int		read_status = 0, write_status;
	char	buf[ CONDOR_IO_BUF_SIZE ];
	int		len;
	int		bytes_to_go;
	int		file_size;
	int scm;
	int rval;
	condor_errno_t terrno;
	sigset_t    omask;


	scm = SetSyscalls( SYS_LOCAL | SYS_UNMAPPED );
	omask = _condor_signals_disable();

	CurrentSysCall = CONDOR_send_a_file;

		/* Do the local open */
	if( (loc_fd = safe_open_wrapper(local,O_RDONLY)) < 0 ) {
		rval = -1;
		_condor_signals_enable( omask );
		SetSyscalls( scm );
		return rval;
	}

		/* Send over the file name, and mode */
	syscall_sock->encode();
	assert( syscall_sock->code(CurrentSysCall) );
	assert( syscall_sock->code(mode) );
	assert( syscall_sock->code(remote) );
	assert( syscall_sock->end_of_message() );

		/* Grab the result of the remote open */
	syscall_sock->decode();
	assert( syscall_sock->code(rval) );
	if( rval < 0 ) {
		assert( syscall_sock->code(terrno) );
		assert( syscall_sock->end_of_message() );
		rval = -1;
		_condor_signals_enable( omask );
		SetSyscalls( scm );
		errno = (int)terrno;
		return rval;
	}
	assert( syscall_sock->end_of_message() );

		/* Send file length */
	file_size = lseek( loc_fd, 0, 2 );
	lseek( loc_fd, 0, 0 );
	syscall_sock->encode();
	assert( syscall_sock->code(file_size) );

		/* Transfer the data */
	for( bytes_to_go = file_size; bytes_to_go; bytes_to_go -= len ) {
		len = bytes_to_go < sizeof(buf) ? bytes_to_go : sizeof(buf);
		read_status = read( loc_fd, buf, len );
		assert( syscall_sock->code_bytes_bool(buf,len) );
	}
	(void)close( loc_fd );

		/* As a check, re-send the length */
	assert( syscall_sock->code(file_size) );
	assert( syscall_sock->end_of_message() );


		/* Get cumulative status from remote writes */
	syscall_sock->decode();
	assert( syscall_sock->code(write_status) );
	if( write_status < 0 ) {
		assert( syscall_sock->code(terrno) );
		assert( syscall_sock->end_of_message() );
		rval = write_status;
		_condor_signals_enable( omask );
		SetSyscalls( scm );
		errno = (int)terrno;
		return rval;
	}
	assert( syscall_sock->end_of_message() );

		/* If we had a local error on the reads, we had to ignore
		   it until now to avoid getting out of sync with the
		   shadow - now we can deal with it.
		*/
	if( read_status < 0 ) {
		rval = read_status;
		_condor_signals_enable( omask );
		SetSyscalls( scm );
		return rval;
	}

		/* Everything worked - status is cumulative status from
		   remote writes.
		*/
	rval = write_status;
	_condor_signals_enable( omask );
	SetSyscalls( scm );
	return rval;
}

int
REMOTE_CONDOR_get_file(char *remote, char *local, int mode)
{
	int		loc_fd;
	int		write_status = 0, read_status;
	char	buf[ CONDOR_IO_BUF_SIZE ];
	int		checksum;
	size_t		len;
	size_t		file_size;
	size_t		bytes_to_go;
	int		scm;
	int		close_status;
	int		rval;
	sigset_t	omask;
	condor_errno_t	terrno;

	scm = SetSyscalls( SYS_LOCAL | SYS_UNMAPPED );
	omask = _condor_signals_disable();

	CurrentSysCall = CONDOR_get_file;

		/* Open local file for writing */
	loc_fd = safe_open_wrapper( local, O_WRONLY|O_CREAT|O_TRUNC, mode );
	if( loc_fd < 0 ) {
		rval = -1;
		_condor_signals_enable( omask );
		SetSyscalls( scm );
		return rval;
	}

		/* Send over the file name */
	syscall_sock->encode();
	assert( syscall_sock->code(CurrentSysCall) );
	assert( syscall_sock->code(remote) );
	assert( syscall_sock->end_of_message() );

		/* Grab the result of the remote open */
	syscall_sock->decode();
	assert( syscall_sock->code(rval) );
	if( rval < 0 ) {
		assert( syscall_sock->code(terrno) );
		assert( syscall_sock->end_of_message() );
		rval = -1;
		_condor_signals_enable( omask );
		SetSyscalls( scm );
		errno = (int)terrno;
		return rval;
	}

		/* Get the size of the file */
	assert( syscall_sock->code(file_size) );

		/* Transfer the data */
	for( bytes_to_go = file_size; bytes_to_go; bytes_to_go -= len ) {
		len = bytes_to_go < sizeof(buf) ? bytes_to_go : sizeof(buf);
		assert( syscall_sock->code_bytes_bool(buf,len) );
		write_status = write( loc_fd, buf, len );
	}
	close_status = close( loc_fd );
	if( close_status < 0 ) {
		dprintf( D_ALWAYS,
			"Close of \"%s\" failed - errno = %d\n", local, errno
		);
	}

		/* Should get file size again as a check */
	assert( syscall_sock->code(checksum) );
	assert( checksum == file_size );

		/* Get status from remote reads */
	assert( syscall_sock->code(read_status) );
	if( read_status < 0 ) {
		assert( syscall_sock->code(terrno) );
		assert( syscall_sock->end_of_message() );
		rval = -1;
		_condor_signals_enable( omask );
		SetSyscalls( scm );
		errno = (int)terrno;
		return rval;
	}
	assert( syscall_sock->end_of_message() );

	/* Wrapup */
	if( write_status >= 0 && read_status >= 0 && close_status >= 0 ) {
		dprintf( D_ALWAYS,
			"CONDOR_get_file: transferred %lu bytes\n", 
			(unsigned long)file_size );
		rval = 0;
	} else {
		rval = -1;
	}

	_condor_signals_enable( omask );
	SetSyscalls( scm );
	return rval;
}

} /* end of extern "C" from prologue */
